--[[
Name: Auction-1.0
Revision: $Rev: 48269 $
Author(s): Shamino (Shamino.wow@gmail.com)
Website: http://www.wowace.com/index.php/Auction-1.0
Documentation: http://www.wowace.com/index.php/Auction-1.0
SVN: http://svn.wowace.com/wowace/trunk/Auction-1.0
Description: Provides a library for accessing auction data and pricing from various Auction Data mods
Dependencies: AceLibrary, AceEvent-2.0
--]]

local MAJOR_VERSION = "Auction-1.0"
local MINOR_VERSION = "$Revision: 48269 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local AuctionLib = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0")

function AuctionLib:OnInitialize()
	self:RegisterEvent("ADDON_LOADED");
end

--find supported addons.  if one is found, automatically set it to be used.
-- it will stop looking after the first one is found
function AuctionLib:ADDON_LOADED()
	if (not self.autofound) then
		for k, v in pairs(self.supportedAddons) do
			if (k == arg1) then
				self:SetAuctionAddon(arg1);
				self.autofound=true;
				self:UnregisterEvent("ADDON_LOADED");
			end
		end
	end
end

function AuctionLib:GetSupportedAuctionAddons()
	return self.supportedAddons;
end

function AuctionLib:SetAuctionAddon(addon)
	if (IsAddOnLoaded(addon) and self.supportedAddons[addon]) then
		self.AuctionAddon=addon;
		return true;
	else
		self:error("Addon '%q' is not supported", addon);
		return false;
	end
end

function AuctionLib:GetAuctionAddon()
	return self.AuctionAddon;
end


function AuctionLib:GetPrice(itemLink)
	if not itemLink then return end
	
	--todo, raise error. auction addon needs to be established
	if not self.AuctionAddon then 
		self:error("Auction Addon has not yet been set.  Use SetAuctionAddon");
		return 
	end

	return self:pcall(self.supportedAddonsFunctions[self.AuctionAddon], itemLink)
end

function AuctionLib:GetAuctioneerPrice(itemLink)
	   local itemID, randomProp, enchant = EnhTooltip.BreakLink(itemLink);
	   local itemKey = itemID..":"..randomProp..":"..enchant;	

	   local medianPrice, medianCount;

	   if (Auctioneer and Auctioneer.Statistic and Auctioneer.Statistic.GetUsableMedian) then
		medianPrice, medianCount = Auctioneer.Statistic.GetUsableMedian(itemKey);
	   else 
		return nil;
	   end

	   if (not medianCount or not medianPrice) then
		return nil;
	   end

	   if (medianCount < 5 or medianPrice == 0) then
		return nil;
	   else
		return medianPrice;
	   end
end


function AuctionLib:GetKCItemsPrice(itemLink)
	local code = KC_Common:GetCode(itemLink);

	--kc_items auctiondb format
	--seen, avgstack, min,   sBidcount, bid, sBuycount, buy
	--4:    5:        10683: 0:	    0:   4:         11208

	local seen, avgstack, min, sBidcount, bid, sBuycount, buy = KC_Auction:GetItemData(code);

	if (min and min ~= 0) then
		return min;
	else 
		return nil;
	end
end

function AuctionLib:GetAuctionMatrixPrice(itemLink)
	local name = AuctionLib_NameFromLink(itemLink);
	local medianPrice = AM_GetMedian(name);

	if (medianPrice) then
		return medianPrice;
	else 
		return nil;
	end
end


function AuctionLib:GetWowEconPrice(itemLink)
	--integer:auction_price in copper, integer:auction_volume, bool:server-specific data
	local auction_price, auction_volume = WOWEcon_GetAuctionPrice_ByLink(itemLink);

	if (auction_price) then
		return auction_price;
	else 
		return nil;
	end
end

local function AuctionLib_NameFromLink(link)
	local itemName = GetItemInfo(link);
	return itemName;
end

local function activate(self, oldLib, oldDeactivate)
	--used to listen for addons being loaded.  will be true if one of the supported addons is found via ADDON_LOADED event
	self.autofound=false;

	--setup the supported auction addons
	self.supportedAddons={
		Auctioneer=true,
		KC_Items=true,
--		AuctionMatrix=true,
		WOWEcon_PriceMod=true,
	}
	self.supportedAddonsFunctions={
		Auctioneer = function(itemLink)
			return self:GetAuctioneerPrice(itemLink)
		end,
		KC_Items = function(itemLink)
			return self:GetKCItemsPrice(itemLink)
		end,
--		AuctionMatrix = function(itemLink)
--			return self:GetAuctionMatrixPrice(itemLink)
--		end,
		WOWEcon_PriceMod = function(itemLink)
			return self:GetWowEconPrice(itemLink)
		end,
	}

	if oldLib then -- if upgrading
		self.AuctionAddon = oldLib.AuctionAddon
		self.autofound = oldLib.autofound
	end

	if oldDeactivate then -- clean up the old library
		oldDeactivate(oldLib)
	end
end

AceLibrary:Register(AuctionLib, MAJOR_VERSION, MINOR_VERSION, activate)
AuctionLib = nil
